CREATE OR REPLACE PACKAGE NUAPPS_CAS is
-------- $Id: PACKAGE_NUAPPS_CAS.sql,v 1.9 2005-11-08 13:51:39+09 naito Exp naito $
--------------------------------------
--  CAS client の呼び出し仕様：
----------------------------------------------------
--  parameter := nuapps_cas.URLparameterTable() ;
--  parameter.extend(N) ; N はパラメタの個数.
--  parameter(1).param := 'text1'     ; parameter(1).value := text1     ;
--  parameter(2).param := 'textarea0' ; parameter(2).value := textarea0 ;
--  parameter(3).param := 'textarea1' ; parameter(3).value := textarea1 ;
--  rc := nuapps_cas.do_login_by_param(ticket, parameter) ;
----------------------------------------------------
--  ここで, rc.status が SUCCESS の時のみ, 実際の表示を行う.
--  parameter(i).value は encode の必要がなく, 
--  do_login_by_param の呼び出し後も decode の必要はない.
--  (do_login_by_param 内で encode/decode が行われている)
---------------------------------------------------
  SUCCESS  constant binary_integer := 0;
  REDIRECT constant binary_integer := 1;
  ENCODING constant nvarchar2(6) := 'UTF-8' ;

  type URLparameter is record (
       param	    nvarchar2(32767), 
       value	    nvarchar2(32767)
       ) ;
  type URLparameterTable is table of URLparameter ;

--   cas_failed exception;
  type casrc is record
    (
    status	binary_integer,
    netid	nvarchar2(32767), 
    uid		nvarchar2(32767), 
    fullName	nvarchar2(32767),
    nagIdNo	nvarchar2(32767), 
    mail	nvarchar2(32767),
    nextticket	nvarchar2(32767),
    fullNameJ	nvarchar2(32767),
    result	nvarchar2(32767), 
    failurecode nvarchar2(32767)
    );

  function do_login_3 (ticket           in nvarchar2 default null,
		      url_in            in nvarchar2 default null, 
		      service           in nvarchar2 default null,
		      walletpath        in nvarchar2 default null,
		      walletpassword    in nvarchar2 default null,
		      renew             in boolean default false)
		      return casrc;

  function do_login_by_param (ticket           in nvarchar2 default null,
			     parameter in out URLparameterTable)
			     return casrc;

  function findValue(parameterName in nvarchar2,
                     parameter in URLparameterTable)
		     return nvarchar2 ;

  procedure redirect_to_login_by_post(login_url in nvarchar2, 
				      ticket	in nvarchar2, 
                                      parameter in URLparameterTable) ;

--  procedure redirect_by_post (url in nvarchar2) ;

  procedure do_logout;

  procedure set_validate_url (url in nvarchar2);
  pragma restrict_references (set_validate_url, wnds, rnds, rnps);

  function  get_validate_url
    return nvarchar2;
  pragma restrict_references (get_validate_url, wnds, rnds, wnps);

  procedure set_login_url (url in nvarchar2);
  pragma restrict_references (set_login_url, wnds, rnds, rnps);

  function  get_login_url
    return nvarchar2;
  pragma restrict_references (get_login_url, wnds, rnds, wnps);

  procedure set_logout_url (url in nvarchar2);
  pragma restrict_references (set_logout_url, wnds, rnds, rnps);

  function  get_logout_url
    return nvarchar2;
  pragma restrict_references (get_logout_url, wnds, rnds, wnps);

  procedure set_service_url (url in nvarchar2);
  pragma restrict_references (set_service_url, wnds, rnds, rnps);

  procedure set_goback_url (url in nvarchar2);
  pragma restrict_references (set_goback_url, wnds, rnds, rnps);

  function  get_service_url
    return nvarchar2;
  pragma restrict_references (get_service_url, wnds, rnds, wnps);

  procedure set_wallet_config (path in nvarchar2, password in nvarchar2);
  pragma restrict_references (set_wallet_config, wnds, rnds, rnps);
  
  procedure set_accepted_proxy (proxy_url in nvarchar2);
  pragma restrict_references (set_accepted_proxy, wnds, rnds, rnps);

  function get_xml_value (tag in nvarchar2, string in nvarchar2) return nvarchar2;
  function xml_analysis(return_data in nvarchar2) return casrc ;
  -- Added by Hisashi NAITO

end nuapps_cas;
/

CREATE OR REPLACE PACKAGE BODY NUAPPS_CAS is
  --
  -- Package-level static data
  --
------- mynu08
 static_validate_url    nvarchar2(2000) := 'http://app.mynu.jp/newcas/serviceValidate';
 static_login_url       nvarchar2(2000) := 'https://app.mynu.jp/newcas/login' ;
 static_logout_url      nvarchar2(2000) := 'https://app.mynu.jp/newcas/logout';
 static_goback_url      nvarchar2(2000) := 'https://app.mynu.jp/newcas/backService.jsp';
 static_sname           nvarchar2(2000) := 'app.mynu.jp' ;
 static_sport           number         := 443 ;
 static_scheme          nvarchar2(2000) := 'https' ; 
-------
  static_proxy           nvarchar2(2000) := null;
  static_service_url     nvarchar2(2000) := null;
  static_wallet_path     nvarchar2(255)  := 'file:/etc/ORACLE/WALLETS/oracle';
  static_wallet_password nvarchar2(255)  := 'oracle';

  --
  -- Our complete collection of accessor functions
  --

  procedure set_validate_url (url in nvarchar2) is
  begin
    static_validate_url := url;
  end set_validate_url;

  function get_validate_url
    return nvarchar2 is
  begin
    return static_validate_url;
  end get_validate_url;

  procedure set_login_url (url in nvarchar2) is
  begin
    static_login_url := url;
  end set_login_url;

  function get_login_url
    return nvarchar2 is
  begin
    return static_login_url;
  end get_login_url;

  procedure set_logout_url (url in nvarchar2) is
  begin
    static_logout_url := url;
  end set_logout_url;

  function get_logout_url
    return nvarchar2 is
  begin
    return static_logout_url;
  end get_logout_url;

  procedure set_service_url (url in nvarchar2) is
  begin
    static_service_url := url ;
  end set_service_url;

  procedure set_goback_url (url in nvarchar2) is
  begin
    static_goback_url := url ;
  end set_goback_url;

  function get_service_url
    return nvarchar2 is
  begin
    return static_service_url;
  end get_service_url;

  procedure set_wallet_config (path in nvarchar2, password in nvarchar2) is
  begin
    static_wallet_path     := path;
    static_wallet_password := password;
  end set_wallet_config;

  procedure set_accepted_proxy (proxy_url in nvarchar2) is
  begin
    static_proxy := proxy_url;
  end set_accepted_proxy;

--- ------------------------------------------
--- Utiliti Functions (by Hisashi NAITO)
--- ------------------------------------------

  --$ url_in の中の "&"||"ticket=..." または "?ticket=..." を削る.
  function delete_ticket_parameter (url_in in nvarchar2) 
    return nvarchar2 
  is
    url_out	nvarchar2(32767) := null ;
    url_head	nvarchar2(32767) ;
    url_tail	nvarchar2(32767) ;
    n0		number ;
    n1		number ;
    n2		number ;
  begin
    n0 := instr(url_in, '&'||'ticket=') ;
    if n0 is null then 
      n0 := instr(url_in, '?'||'ticket=') ;
      if n0 is null then 
        url_out := url_in ;
	return url_out ;
	-- "&"||"ticket=" も "?"||"ticket=" も見つからない.
	-- そのまま返す.
      end if;
    end if;
    -- この時点で n0 は "&" || "ticket=" または "?ticket=" の先頭.
    url_head := substr(url_in, 0, n0) ;
    url_tail := substr(url_in, n0+1) ;
    -- url_head は ".ticket=" の直前まで.
    -- url_tail は "ticket=" で始る.
    n1 := instr(url_tail, '&') ;
    if n1 is null then 
    -- 次のパラメータが見つからない.
      return url_head ;
    end if;
    url_tail := substr(url_tail, n1+1) ;
    url_out := url_head || url_tail ;
    return url_out;
  end delete_ticket_parameter ;



  -- CGI 環境の QUERY_STRING を受け取って, "ticket" を取り除く
  -- さらに, "&" -> "%26", "?" -> "%3f" に変換した文字列を返す.
  function make_esc_query_string (in_string in nvarchar2) 
    return nvarchar2 
  is
    s_index	number := 1 ;
    n1		number;
    n2		number;
    n3		number;
    l		number;
    k		number;
    return_str	nvarchar2(32767) := null ;
    url_key	nvarchar2(32767);
    url_value	nvarchar2(32767);
  begin

    l := length(in_string) ;
    if l is null then 
      return null ;
    end if ;
--- FOR DEBUG
---    return null ;
--- END FOR DEBUG

    LOOP
      n1 := instr(in_string, '&', s_index) ;
      if n1 = 0 then			 -- '&' がこれ以上みつからない -> 終了
	EXIT;
      else		  		  -- '&' が見つかった.
	n2 := instr(in_string, '=', n1) ; -- n2 は 0 にならないはず: '=' は存在するのが URL 規約.
	n3 := instr(in_string, '&', n2) ; -- 次の '&' の直前までを切り出す.
        url_key   := substr(in_string, n1+1, n2-n1) ;
	if url_key != 'ticket' then               -- key が "ticket" の時は無視
	  if n3 = 0 or n3 is null then			  -- 次の '&' が見つかった.
	    s_index := n2+1;
            url_value := substr(in_string, n2+1);
	  else
	    s_index := n3 ;
	    url_value := substr(in_string, n2+1, n3-n2-1) ;
	  end if;
--          return_str := return_str || '%26' || url_key || '=' || url_value ;
          return_str := return_str || '%26' || url_key || url_value ;
        end if;
      end if;
    end LOOP;
    return_str := replace(return_str, '%3f%26', '%3f') ;
    return return_str ;
  end make_esc_query_string;


  -- static_service_url を構成する.
  -- 最終変更： 2005/02/01
  function make_static_service_url (dummy in nvarchar2) 
    return nvarchar2 
  is
    s_url   nvarchar2(32767);
    sport   nvarchar2(32767);
    sname   nvarchar2(32767);
    script  nvarchar2(32767);
    scheme  nvarchar2(32767) := 'http';
  begin
    -------------------
--     --* changed by H.NAITO
--     sname := static_sname ;
--     sport := static_sport ;
--     scheme := static_scheme ; 
--     --* End of chanegd by H.NAITO
    -------------------
    sport  := static_sport ;
    scheme := static_scheme ;
    sname  := static_sname ;
    if static_sport is null then
      sport := owa_util.get_cgi_env('SERVER_PORT') ;
      if static_scheme is null then
        if sport = '443' then 
	  scheme := 'https' ;
	else 
          if sport = '80' then 
	    scheme := 'http' ;
          end if ;
	end if ;
      end if ;
    end if ;
    if static_sname is null then
      sname := owa_util.get_cgi_env('SERVER_NAME') ;
    end if ;
    -------------------
    script := owa_util.get_cgi_env('SCRIPT_NAME');
    if sport = '443' or sport = '80' then
     s_url := scheme || '://' || sname || script || '/' || lower(owa_util.get_procedure) ;
    else 
     s_url := scheme || '://' || sname || ':' || sport || script || '/' || lower(owa_util.get_procedure) ;
    end if;
    return s_url ;
  end make_static_service_url ;

--- ------------------------------------------
--- LOGOUT
---   CALL logout servlet
--- ------------------------------------------

  procedure do_logout is
  begin
    owa_util.redirect_url(static_logout_url);
  end do_logout;

--- ------------------------------------------
--- LOGIN
--- ------------------------------------------

------------- login への redirect の書き換え
------------- url は URL "Q" XXXX "A" YYYY....
------------- となっている.
------------- これを POST を行う JavaScript を含むコードに変換する.
------------- ENCODING は "ENCODING" parameter に含まれるものを利用する.
  procedure redirect_by_post(login_url in nvarchar2,
		             ticket    in nvarchar2,
                             service   in nvarchar2, 
                             method    in nvarchar2, 
                             encoding  in nvarchar2)
  is 
    form_string	nvarchar2(32767) := null ;
  begin
    -- 次に, URL parameter を順番に "parameter" から探す.
    form_string := chr(10) || '<form name="form" action="' || login_url || '" method=POST>' || chr(10) ;
    form_string := form_string || '<input type="hidden" name="ticket"  value="' || ticket  || '">' || chr(10) ;
    form_string := form_string || '<input type="hidden" name="service" value="' || service || '">' || chr(10) ;
    form_string := form_string || '<input type="hidden" name="CASREQUESTMETHOD" value="' || method || '">' || chr(10) ;
    form_string := form_string || '<input type="hidden" name="ENCODING" value="' || encoding || '">' || chr(10) ;
    form_string := form_string || '<noscript><input type="submit"></noscript>' || chr(10)  ;
    form_string := form_string || '</form>'  || chr(10)  ;
    -- ここまで
    htp.p('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">') ;
    htp.p('<html>') ;
    htp.p('<head>') ;
    htp.p('<title>Central Authentication Service</title>') ;
    htp.p('<meta http-equiv="Content-Type" content="text/html;charset=' || encoding || '">') ;
    htp.p('</head>') ;
    htp.p('<body bgcolor="#ffffff" link="#000000" alink="#000000" vlink="#000000" onLoad="document.form.submit()">') ;
    htp.p('<noscript>Push </noscript>') ;
    htp.p(form_string) ;
    htp.p('<noscript>to access the service you requested.</noscript>') ;
    htp.p('</body>') ;
    htp.p('</html>') ;
    return ;
  end redirect_by_post ;

  --* 
  -- string の中の <cas:tag>xxxx</cas:tag> の xxxx を返す.
  --*
  function get_xml_value (tag in nvarchar2, string in nvarchar2) 
    return nvarchar2
  is
    startTag	nvarchar2(32767) ;
    endTag	nvarchar2(32767) ;
    returnS	nvarchar2(32767) ;
    n1		integer ;
    n2		integer ;
  begin 
    startTag := '<cas:' || tag || '>' ;
    endTag   := '</cas:' || tag || '>' ;
      n1 := instr(string, startTag) + length(startTag) ;
      n2 := instr(string, endTag) ;
      returnS := substr(string, n1, n2-n1) ;
      return returnS ;
  end get_xml_value ;

  --* xml data を解析
--- -----------------------------
--- 返ってくる情報は以下の通り：
--- 成功したとき：
---   <cas:serviceResponse xmlns:cas='http://www.yale.edu/tp/cas'>
---     <cas:authenticationSuccess>
---       <cas:ticket>ST-68-57i444BkNQLINhHcMTxP</cas:ticket>
---       <cas:user>naito</cas:user>
---       <cas:RemoteAddr>172.16.254.254</cas:RemoteAddr>
---       <cas:dn>uid=naito,ou=people,dc=math,dc=nagoya-u,dc=ac,dc=jp</cas:dn>
---       <cas:ServerName>srvlet.math.nagoya-u.ac.jp</cas:ServerName>
---       <cas:RemoteHost>rabbit-172</cas:RemoteHost>
---       <cas:Attributes>
---         <cas:uid>naito</cas:uid>
---         <cas:registMailAddress>naito@math.nagoya-u.ac.jp</cas:registMailAddress>
---         <cas:nagFullName-Roman>Hisashi Naito</cas:nagFullName-Roman>
---       </cas:Attributes>
---     </cas:authenticationSuccess>
---   </cas:serviceResponse>
---
--- 失敗したとき：
---  <cas:serviceResponse xmlns:cas='http://www.yale.edu/tp/cas'>
---    <cas:authenticationFailure code='CODE'>
---      errorMessage
---    </cas:authenticationFailure>
---  </cas:serviceResponse>
---
---  CODE: INVALID_TICKET のとき
---        redirect to LOGIN SERVLET with TICKET=NULL, SERVICE
---  CODE: otherwise
---        redirect to goBack servlet
---
--- -----------------------------
  function xml_analysis(return_data in nvarchar2) 
    return casrc
  is
    rc			casrc;
    tempstring		nvarchar2(32767) := null ;
    failurecode		nvarchar2(32767) := null ;
    n0			number ;
    n1			number ;
    n2			number ;
    n3			number ;
    n4			number ;
  begin 
    n0 := instr(return_data,'<cas:authenticationSuccess>');
    if n0 != 0 then
    --* Authentication Success の場合 ------------------------------------
    --* tempsring は <cas:Attributes> -- </cas:Attributes> の中身
      rc.status      := SUCCESS ;
      rc.nextticket  := get_xml_value('ticket', return_data) ;
      tempstring     := get_xml_value('Attributes', return_data) ;
      rc.mail        := get_xml_value('registMailAddress', tempstring) ;
      rc.fullnameJ   := get_xml_value('fullName;lang-ja', tempstring) ;
      rc.fullname    := get_xml_value('fullName;lang-en', tempstring) ;
      rc.netid       := get_xml_value('nagIdNo', tempstring) ;
      rc.uid         := get_xml_value('uid', tempstring) ;
      rc.nid         := get_xml_value('nagoyaunivid', tempstring) ;
      rc.status      := SUCCESS ;


    else
    --* Authentication Failure の場合 ---------------------------------
    -- got a response but authentication failed - will return REDIRECT status
    --* get Failure_Code 
      n2 := instr(return_data, '<cas:authenticationFailure code=') + length('<cas:authenticationFailure code=') + 1 ;
      tempstring := substr(return_data,n2) ;
      n3 := instr(tempstring, '>') - 1 ;
      failurecode := substr(tempstring, 0, n3-1) ;
      -- End of get Failure_Code
      --
      --* ? (failurecode is INVALID_TICKET)
      n4 := instr(failurecode, 'INVALID_TICKET') ;
      if n4 != 0 then
        rc.failurecode := 'INVALID_TICKET' ;
	rc.status := REDIRECT;
	rc.netid  := null;
      else
	--* otherwise => redirect goBack
	rc.failurecode := 'REDIRECT' ;
        rc.status := REDIRECT;
        rc.netid  := null;
      end if;
    end if;
    return rc ;
  end xml_analysis ;

  -----------------------------------------------
  -- ticket のチェックを行う.
  -- 最終変更： 2005/02/01
  -----------------------------------------------
--   【動作】
--      1) ticket が null である場合.
--         => login servlet へ redirect (by using POST method)
--      2) ticket が non-null である場合.
--         => validation servlet との通信を行い, 
--   	   ticket validation の結果を得る.
--      3) ticket が valid の場合.
--         => return SUCCESS
--      4) ticket が INVALID TICKET の場合.
--         => login servlet へ redirect (by using POST method)
--      5) ticket がその他のエラーの場合.
--         => goback servlet へ redirect (by using GET method)
--   【引数の扱い】
--      1) service argument から svc を取り出す.
--         example:
--           svc = https://app.mynu.jp/KYKR0010
--      2) service argument から parameter を取り出す.
--         example:
--           parameter := key_1 '&' value_1 '&' .... '&' key_n=value_n
--      3) CASREQUESTMETHOD=POST と ENCODING=SHIFT_JIS をセットする.
--      A) validation の方法：
--         validation servlet に対して "svc" と ticket を渡す.
--         example:
--   	  validation_servlet '&' ticket=TICKET '&' service=svc ;
--      B) login への redirect の方法：
--         login へ以下の情報を POST method で渡す.
--         svc, と各パラメータ.
  -----------------------------------------------
  function do_login_3 (ticket in nvarchar2 default null,
		      url_in in nvarchar2 default null, 
		      service in nvarchar2 default null,
		      walletpath in nvarchar2 default null,
		      walletpassword in nvarchar2 default null,
		      renew in boolean default false)
    return casrc 
  is
    rc			casrc;
    service_parameter   nvarchar2(32767) := null ;
    parameter		nvarchar2(32767) := null ;
    parameter_for_login nvarchar2(32767) := null ;
    parameter_for_valid nvarchar2(32767) := null ;
    esc_query_string	nvarchar2(32767) := null ;
    in_url_in		nvarchar2(32767) := null ;
    temp_url_in		nvarchar2(32767) := null ;
    new_url_in		nvarchar2(32767) := null ;
    validate_url	nvarchar2(32767) := null ;
    goback_url		nvarchar2(32767) := null ;
    newsvc_url		nvarchar2(32767) := null ;
    return_data		nvarchar2(32767) := null ;
    request_method	nvarchar2(32767) ;
    n0			number ;
    n1			number ;
  begin
    --   static_service_url を作成する.
    if static_service_url is null then
       static_service_url := make_static_service_url('DUMMY') ;
    end if;
    -- url_in から service 部分と ticket 部分を切り出す.
    -- service_parameter は SERVICE URL 部分
    -- parameter は prefix "ticket=xxx '&' " までを除いた suffix
    service_parameter := url_in ;
    n0 := instr(service_parameter, '?') ;
    if n0 is not null then
      service_parameter := substr(service_parameter, 0, n0-1) ;
    end if ;
    parameter := substr(url_in, n0+1) ;
    n0 := instr(parameter, 'ticket=') + length('ticket=') ;
    n0 := instr(parameter, '&', n0) ;
    parameter := substr(parameter, n0+1) ;
    --
    if length(parameter) > 0 then
      parameter_for_login := static_service_url || '?' || parameter ;
    else 
      parameter_for_login := static_service_url ;
    end if;
    --*
    --  If ticket does not exists, then access to login
    --  login にアクセスするとき, 長いパラメータを使えるようにする.
    if ticket is null then
      if length(parameter) > 0 then
        redirect_by_post(static_login_url, 'null', parameter_for_login, 'POST', ENCODING) ;
      else 
        owa_util.redirect_url(static_login_url || '?' || 'service=' || static_service_url);
      end if;
      rc.status := REDIRECT ;
      return rc ;
    end if;
    
    --* reconstruct the "parameter": need TICKET and SERVICE
    parameter_for_valid := static_validate_url || '?ticket=' || ticket || '&' || 'service=' || static_service_url ;
    --* 実際に validation servlet と通信する.
    if walletpath is null or walletpassword is null then
      return_data := utl_http.request(parameter_for_valid, null, static_wallet_path, static_wallet_password);
    else
      return_data := utl_http.request(parameter_for_valid, null, walletpath, walletpassword);
    end if;
    --   ここまで, service validation servlet との通信

    --* return_data の解析
    rc := xml_analysis(return_data) ;
    rc.result := return_data || '' ;
    rc.result := return_data || '' ;

    --* success の場合には単に rc を返せばよい.
    if rc.status != SUCCESS then 
      if rc.failurecode = 'INVALID_TICKET' then 
        if length(parameter) > 0 then
          redirect_by_post(static_login_url, 'null', parameter_for_login, 'POST', ENCODING) ;
        else 
          owa_util.redirect_url(static_login_url || '?' || 'service=' || static_service_url);
        end if;
      else 
        goback_url := static_goback_url|| '?edu.yale.its.tp.cas.status=' || rc.failurecode;
        owa_util.redirect_url(goback_url);
      end if ;
    end if ;
    return rc ;
  end do_login_3;

  --* ticket, parameters -> CAS perform!
  --* parameter is encode then decode if CAS perfom is success.
  function do_login_by_param (ticket in nvarchar2 default null,
			     parameter in out URLparameterTable)
    return casrc 
  is
    rc			casrc;
    p			URLparameter ;
    parameter_for_valid nvarchar2(32767) := null ;
    return_data		nvarchar2(32767) := null ;
    goback_url		nvarchar2(32767) := null ;
    i			integer ;
    n			integer ;
  begin
    --* IF ticket is NULL, then redirect to LOGIN by POST and RETURN.
    if ticket is null then
      redirect_to_login_by_post(static_login_url, ticket, parameter) ;
      rc.status := REDIRECT ;
      return rc ;
    end if;
    -- 

    --* IF ticket is NOT NULL, then communicate VALIDATION and get XML.
    if static_service_url is null then static_service_url := make_static_service_url('DUMMY') ; end if;
    parameter_for_valid := static_validate_url || '?ticket=' || ticket || '&' || 'service=' || static_service_url ;
    return_data := utl_http.request(parameter_for_valid, null, static_wallet_path, static_wallet_password);
    -- 

    --* return_data の解析
    rc := xml_analysis(return_data) ;
    rc.result := return_data || '' ;
    rc.result := return_data || '' ;

    --* success の場合には単に rc を返せばよい.
    --* if success then, decode the parameter.value.
    if rc.status != SUCCESS then 
      if rc.failurecode = 'INVALID_TICKET' then 
        redirect_to_login_by_post(static_login_url, ticket, parameter) ;
      else 
        goback_url := static_goback_url|| '?edu.yale.its.tp.cas.status=' || rc.failurecode;
        owa_util.redirect_url(goback_url);
      end if ;
      rc.status := REDIRECT ;
    else 
      if parameter.last > 0 then 
        for i in 1 .. parameter.last 
        LOOP 
          parameter(i).value := url_encode.toEntity(parameter(i).value) ;
        END LOOP ;
      end if ;
    end if ;
    return rc ;
  end do_login_by_param ;

  --* redirect to LOGIN by POST using parameter by JavaScript
  procedure redirect_to_login_by_post(login_url in nvarchar2, 
                                      ticket	in nvarchar2, 
                                      parameter in URLparameterTable) 
  is 
    n	integer ;
    i	integer ;
    p	URLparameter ;
  begin
    n := parameter.last ;
    if static_service_url is null then static_service_url := make_static_service_url('DUMMY') ; end if;
    htp.p('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">') ;
    htp.p('<html>') ;
    htp.p('<head>') ;
    htp.p('<title>Central Authentication Service</title>') ;
    htp.p('<meta http-equiv="Content-Type" content="text/html;charset=' || encoding || '">') ;
    htp.p('</head>') ;
    htp.print('<body bgcolor="#ffffff" link="#000000" alink="#000000" vlink="#000000" onLoad="document.form.submit()">') ;
    htp.print('<noscript>Push </noscript>') ;
    htp.print('<form name="form" action="' || login_url || '" method=POST>') ;
    htp.print('<input type="hidden" name="ticket"  value="null">') ;
    htp.prn('<input type="hidden" name="service" value="' || static_service_url) ;
    if n >= 1 then 
      p := parameter(1) ;
      htp.prn('?' || p.param || '=') ;
      htp.prn(url_encode.encode(p.value)) ;
      for i in 2 .. n
      LOOP
        p := parameter(i) ;
        htp.prn('&' || p.param || '=') ;
	htp.prn(url_encode.encode(p.value)) ;
      END LOOP ;
    end if ;    
    htp.print('">') ;
    htp.print('<input type="hidden" name="CASREQUESTMETHOD" value="POST">') ;
    htp.print('<input type="hidden" name="ENCODING" value="' || ENCODING || '">') ;
    htp.print('<noscript><input type="submit"></noscript>') ;
    htp.print('</form>') ;
    htp.p('<noscript>to access the service you requested.</noscript>') ;
    htp.bodyClose ;
    htp.htmlClose ;
    return ;
  end redirect_to_login_by_post ;

  --* parameter の中から key が "parameterName" になる value を返す.
  function findValue(parameterName in nvarchar2,
                     parameter in URLparameterTable)
    return nvarchar2 
  is
    i	integer ;
  begin 
    for i in 1 .. parameter.last 
    LOOP
      if parameter(i).param = parameterName then 
        return parameter(i).value ;
      end if ;
    END LOOP ;
    return null ;
  end findValue ;
  

end nuapps_cas;
/
