CREATE OR REPLACE PACKAGE URL_ENCODE is
-------- $Id: PACKAGE_URL_ENCODE.sql,v 1.18 2005-11-14 15:51:25+09 naito Exp naito $
-------- 0-9A-z はそのまま. ".", "-", "*", "_" もそのまま.
-------- SPACE は +, 後は encode.
-------- $Id: PACKAGE_URL_ENCODE.sql,v 1.18 2005-11-14 15:51:25+09 naito Exp naito $
-------- URL に含まれた特殊文字を encode/decode する.

  ---- 特殊文字を URL_ENCODE する.
  function encode (parameter in nvarchar2 default null) return nvarchar2;

  ---- URL_ENCODE された特殊文字を DECODE する.
  function decode (parameter in nvarchar2 default null) return nvarchar2;

  ---- 16進 -> 10進変換
  function hextodec (p_hex in nvarchar2) return number ;

  ---- 10進 -> 16進変換
  function dectohex (p_dec in number) return nvarchar2 ;

  ---- ctype functions
  function isalphanumeric (c in number) return number ;
  function isnotencode (c in number) return number ;
  function isutf8 (c in number) return number ;

  function toURL (s in nvarchar2, c in number) return nvarchar2 ;
  function toChar(s_hex in nvarchar2 default null) return nvarchar2 ;
  function toEntity (s in nvarchar2) return nvarchar2 ;

end url_encode ;
/

CREATE OR REPLACE PACKAGE BODY URL_ENCODE is

  --*
  -- 文字列 parameter に対して,
  -- その URL encode 結果を返す.
  --*
  function encode (parameter in nvarchar2 default null)
    return nvarchar2 
  is
    return_string nvarchar2(32767) ;
    l number ;
    i number ;
    s nvarchar2(2000) ;
    p nvarchar2(32767) ;
    c number ; 
  begin
    return_string := parameter ;
    l := length(parameter) ;
    if l is null then 
      return parameter ;
    end if ;
    if l = 0 then 
      return parameter ; 
    end if ;

    for i in 1 .. l 
      LOOP
	s := substr(parameter,i,1) ;
	c := ascii(s) ;
	-- Only isalphanumeric character
--  	if isalphanumeric(c) = 1 then
 	if isnotencode(c) = 1 then
	  p := concat(p, s) ;
        else 
          p := concat(p, toURL(s, c)) ;
	end if ;
      END LOOP ;
    return p ;
  end encode ;

  --*
  -- URL encode された文字列 parameter に対して,
  -- その decode 結果を返す.
  --*
  function decode (parameter in nvarchar2 default null)
    return nvarchar2 
  is
    return_string nvarchar2(32767) ;
    n		  integer ;
    l		  integer ;
    m0		  integer ;
    m1		  integer ;
    j		  integer ;
    c		  varchar2(100) ;
    s		  nvarchar2(32767) ;
    p		  nvarchar2(32767) ;
  begin
--     if parameter is null then 
--       return parameter 
--     end if;

    p := '' ;
    l := length(parameter) ;
    n := 0 ;
    while n < l 
    LOOP 
	-- "%..%..%.." となっている連続する部分文字列を順に検索.
	-- その部分を関数に渡して文字に変換.
        m0 := instr(parameter, '%', n+1) ;
	if m0 > 0 then 
	  p := concat(p, substr(parameter, n+1, m0-n-1) ) ;	-- '%' 以前の文字を p に concat する.
	  -- m0 + 1 以後で連続する '%..' の列の最大長を取得.
	  m1 := m0 ;
	  LOOP
	    j := instr(parameter, '%', m1+1) ;			-- 次の '%' の場所
	    EXIT when j = 0 OR (j - m1) != 3 ;
	    m1 := j ;
	  END LOOP ;
	  if j = 0 then						-- 最後までこの文字列
	    s := substr(parameter, m0) ;
	  else 
	    s := substr(parameter, m0, m1 - m0 + 3) ; 
	  end if ;						-- この時点で s には '%FF%FF....%FF' が入っている.
  	  p := concat(p, toChar(s)) ;
	  n := m1 + 2 ;
	else 
  	  p := concat(p, substr(parameter, n+1) ) ;
	  EXIT ;
	end if;
    END LOOP ;
    return p ;
  end decode ;

  --*
  -- 16 進文字列に対して, その文字を返す
  -- UTF-8 文字に対応： 
  -- UTF-8 文字の先頭バイトは 0xC0 = 192 以上, 
  -- 後続バイトは 0xBF = 191 以下
  --*
  function toChar(s_hex in nvarchar2 default null)
    return nvarchar2
  is
    str0	nvarchar2(32767) ;
    str1	nvarchar2(32767) ;
    len		integer ;
    n		integer ;
    dec		integer ;
    dec0	integer ;
  begin
    len := length(s_hex) ;
    n := 2 ;
    str0 := '' ;
    dec0 := 0 ;
    while n < len 
    LOOP
      str1 := substr(s_hex, n, 2) ;
      n := n+3 ;
      dec  := hextodec(str1) ;
      if dec < 128 then	     -- 読んだ文字が ASCII: これまでの文字を出力して, この文字を出力.
        if dec0 != 0 then 
 	  str0 := concat(str0, chr(dec0)) ;
	  dec0 := 0 ;
	end if ;
	str0 := concat(str0, chr(dec)) ;
      else if dec < 192 then	-- 読んだ文字が UTF-8 第２バイト以降: そのまま継続.
	dec0 := dec0 * 256 + dec ;
      else			 -- 読んだ文字が UTF-8 第１バイト： これまでの文字を出力して, 継続.
        if dec0 != 0 then 
 	  str0 := concat(str0, chr(dec0)) ;
	end if;
        dec0 := dec ;
      end if ; end if;
    END LOOP ;
    if dec0 != 0 then 
      str0 := concat(str0, chr(dec0)) ;
    end if;
    return str0 ;
  end toChar ;

  --*
  -- 16 進文字列に対して, その 10 進値を返す.
  --*
  function hextodec(p_hex in nvarchar2)
    return number 
  is
    vHexString nvarchar2(16) ;
    vHexChar   char(1 char) ;
    vDigit     number ;
    vDecimal   number ;
  begin 
    vDecimal := 0 ;
    vHexString := upper(ltrim(p_hex, '0')) ;
    for i in 1 .. length(vHexString) 
    LOOP
      vHexChar := substr(vHexString, i, 1) ;
      CASE vHexChar
	WHEN 'A' THEN vDigit := 10 ;
	WHEN 'B' THEN vDigit := 11 ;
	WHEN 'C' THEN vDigit := 12 ;
	WHEN 'D' THEN vDigit := 13 ;
	WHEN 'E' THEN vDigit := 14 ;
	WHEN 'F' THEN vDigit := 15 ;
	ELSE	      vDigit := to_number(vHexChar) ;
      END CASE ;
      vDecimal := 16 * vDecimal + vDigit ;
    END LOOP ;
    return vDecimal ;
  end hextodec ;

  --*
  -- 文字コード値 p_dec に対して, その 16 進文字列を返す.
  --*
  function dectohex(p_dec in number) 
    return nvarchar2 
  is 
  begin 
    return to_char(P_DEC,'FMXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX');
  end dectohex ;

  --*
  -- 文字コード値 c に対して,
  -- それが 0-9A-z になっているかどうか.
  -- 0-9A-z の場合には 1 を返し, 
  -- そうでない場合には 0 を返す.
  --*
  function isalphanumeric (c in number) 
    return number 
  is
  begin
    if c >= 48 and c <= 57 or c >= 65 and c <= 90 or c >= 97 and c <=122 then
      return 1 ;
    end if;
    return 0 ;
  end isalphanumeric ;
    
  --*
  -- 文字コード値 c に対して, encode すべき文字かどうか？
  -- エンコードすべき文字の場合には 0 を返し, 
  -- そうでない場合には 1 を返す.
  --*
  function isnotencode (c in number) 
    return number 
  is
  begin
--     if c = 43 then -- '+'
--       return 0 ;
--     end if;
    if c = ascii('&') then
      return 0 ;
    end if;
    if c = ascii('?') then
      return 0 ;
    end if;
    if c = ascii('=') then
      return 0 ;
    end if;
    if c = ascii('"') then
      return 0 ;
    end if;
    if c = ascii('<') then
      return 0 ;
    end if;
    if c = ascii('>') then
      return 0 ;
    end if;
    if c = ascii('%') then
      return 0 ;
    end if;
--     if c = 32 then -- SPACE
--       return 0 ;
--     end if;
    if c < 32 then -- CONTROL CHARACTER
      return 0 ;
    end if;
    return 1 ;
  end isnotencode ;
    
  --*
  -- 文字コード値 c に対して,
  -- それが utf-8 文字かどうか
  -- utf-8 文字の場合には 1 を返し, 
  -- そうでない場合には 0 を返す.
  --*
  function isutf8 (c in number) 
    return number 
  is
  begin
    if c >= 128 then
      return 1 ;
    end if;
    return 0 ;
  end isutf8 ;

  --*
  -- s は「１文字」 (utf-8 の場合は, utf-8 1 character)
  -- s := substr(parameter,i,1) ;
  -- c は s の文字コード値 
  --   utf-8 の場合は, 1 character = 3 bytes ならば, 
  --   3 octet になる.
  -- c := ascii(s) ; 
  --*
  function toURL (s in nvarchar2, c in number) 
    return nvarchar2 
  is
    len integer ;
    n   integer ; 
    t   nvarchar2(100) ;
    u   nvarchar2(100) ;
    p   nvarchar2(100) ;
  begin
    if isutf8(c) = 1 then
    -- utf-8 文字は 1 byte づつ encode する.
      t := dectohex(c) ;
      len := length(t) ;
      n := 1 ;
      p := '' ;
      while n <= len 
      LOOP
        p := concat(p, '%' || substr(t, n, 2)) ;
	n := n + 2 ;
      END LOOP ;
      return p ;
    end if ;
    if c < 16 then
      return  '%' || '0' || dectohex(c) ;
    end if ;
    return '%' || dectohex(c) ;
  end toURL ;
	   
  --*
  -- '&', ">", "<", """ を char entity に変換
  --*
  function toEntity (s in nvarchar2)
    return nvarchar2 
  is
    c	nvarchar2(32767) ;
  begin
    c := s ;
    c := replace(c, '&', '&' || 'amp;') ;
    c := replace(c, '<', '&' || 'lt;') ;
    c := replace(c, '>', '&' || 'gt;') ;
    c := replace(c, '"', '&' || 'quot;') ;
    return c ;
  end toEntity ;
	   
end url_encode ;
/
